//
// Turtle Graphics Library extension for teaching OO concepts
// by Hideki Tsuiki, Kyoto University
// as a modification of Tatsuya Hagino's Turtle Graphics Library
// Copyright (C) 2000, Hideki Tsuiki, Kyoto University

// Turtle Graphics Library for Information Processing I
// Copyright (C) 1998, Tatsuya Hagino, Keio University
//
// Permission to use, copy, modify, and distribute this software
// for educational purpose only is hereby granted, provided that
// the above copyright notice appear in all copies and that both
// the copyright notice and this permission notice appear in
// supporting documentation.  This software is provided "as is"
// with no warranty.


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.Math;
import java.util.Vector;



/** ȥɽȤΥ饹 */
public class TurtlePanel extends JPanel {
  JComponent jc = new JComponent(){  // Anonymous class
    public void paint(Graphics g) {
      synchronized(mappedLock) {
	if (!mapped) {
	  mapped = true;
	  mappedLock.notify();
	}
      }
      if(mesh){
	drawmesh(g);
      }
      for (int i = 0; i < history.size(); i++) {
	Line line = (Line)history.elementAt(i);
	g.setColor(line.c);
	g.drawLine(line.bx, line.by, line.ex, line.ey);
      }
      for (int i = 0; i < turtles.size(); i++) {
	Turtle t = (Turtle)turtles.elementAt(i);
	t.show(g);
      }
    }
  };
  
  Vector turtles = new Vector(10);
  boolean mesh= false;
  Color meshDark = new Color(230,230,100);
  Color meshLight = new Color(230,230,230);

  // turtle segments history
  Vector history;

  // class for line segment
  class Line {
    public int bx;
    public int by;
    public int ex;
    public int ey;
    public Color c;
    public Line(int bx, int by, int ex, int ey, Color c) {
      this.bx = bx;
      this.by = by;
      this.ex = ex;
      this.ey = ey;
      this.c = c;
    }
  }

  // whether frame is mapped or not
  boolean mapped = false;
  // mutex locks
  final Object mappedLock = "mapped lock";

  // menu
  JRadioButtonMenuItem[] speedMenu;
  static String[] speedMenuString = {"no kame", "fast", "normal", "slow"};

  public TurtlePanel(int width, int height) {
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    add(jc);
    setPreferredSize(new Dimension(width, height));
    setBackground(Color.white);
    setForeground(Color.black);
    history = new Vector(10);
  }

  /** TurtlePanel 򡢥ǥեȤ礭(400x400) Ǻ롣 */
  public TurtlePanel() {
    this(400,400);
  }

  /** t 򤳤Υե졼ξɲä롣 */
  public void add(Turtle t)
  {
    turtles.addElement(t);
    t.setFrame(this);
  }

  /** t 򤳤Υե졼फ롣 */
  public void remove(Turtle t)
  {
    turtles.removeElement(t);
    repaint();
  }

  /** ޤǤ˽񤫤줿Ƥä*/
  public void clear(){
     history.clear();
     repaint();
  }

  void addLineElement(int xx, int yy, int x, int y, Color c)
  {
    history.addElement(new Line(xx,yy,x,y,c));
  }

  public void drawmesh(Graphics g)
  {
    int width = getWidth();
    int height = getHeight();
  
    g.setColor(meshLight);
    for(int x = 0; x < width; x+= 10){
      g.drawLine(x,0,x,height);
    }
    for(int x = 0; x < height; x+= 10){
      g.drawLine(0,x,width,x);
    }
    g.setColor(meshDark);
    for(int x = 0; x < width; x+= 50){
      g.drawLine(x,0,x,height);
    }
    for(int x = 0; x < height; x+= 50){
      g.drawLine(0,x,width,x);
    }
  }
  
  public void addMesh() {
    mesh = true;
  }
}


